@@ -1,10 +1,12 @@
 //! Everything related to `libavcodec`.
+mod bitstream;
 mod codec;
 mod codec_id;
 mod codec_par;
 mod packet;
 mod parser;
 
+pub use bitstream::*;
 pub use codec::*;
 pub use codec_id::*;
 pub use codec_par::*;
