@@ -15,8 +15,8 @@ pub enum RsmpegError {
     // --------- Unstablized error type below ------
 
     // FFmpeg errors
-    #[error("Cannot open input file.")]
-    OpenInputError,
+    #[error("Cannot open input file. ({0})")]
+    OpenInputError(c_int),
     #[error("Cannot open output file.")]
     OpenOutputError,
     #[error("Cannot find stream information.")]
@@ -105,8 +105,8 @@ pub enum RsmpegError {
     #[error("AVDictionary failed to get string.")]
     DictionaryGetStringError,
 
-    #[error("AVIO Open failure.")]
-    AVIOOpenError,
+    #[error("AVIO Open failure. ({0})")]
+    AVIOOpenError(i32),
 
     #[error("SwrContext init failed.")]
     SwrContextInitError,
